# -*- coding:Utf-8 -*-
import es

esInfo = es.AddonInfo()
esInfo.author = "DoCky"
esInfo.version = "1.102p"
esInfo.name = "PlayerViewLib"
esInfo.basename = "playerview"
esInfo.url = "http://forums.eventscripts.com/viewtopic.php?t=42292"
esInfo.description = "Allow you to know what entity a player is aiming"

version = es.ServerVar("playerviewlib", "", " Created by DoCky -")
version.set(esInfo.version)
version.makepublic()


# players dict
players = {}

def _command():
    # exec internal i_entity method
    players[es.getargv(1)].i_entity(es.getargv(2))
# register block for internal command
es.addons.registerBlock("_playerviewlib", "_command", _command)
# register server command
es.regcmd("_playerview_cmd", "_playerviewlib/_command", "playerview: internal command")

class PlayerView(object):
    """ PlayerView object """
    def __init__(self, userid, callback = None):
        self.userid = str(userid)
        self.callback = callback
        self.entities = {}
        self.block = None

        # public attributes
        self.entity = -1
        self.player = 0
        self.playerindex = -1

    def i_entities(self):
        # clear dict
        self.entities.clear()
        # for entities in map
        for index in es.createentityindexlist():
            # store "index" = "targetname"
            self.entities[index] = es.entitygetvalue(index, "targetname")

    def i_entity(self, target):
        # default index value
        entity = -1
        # targetname to find
        targetname = "_playerview_%s_%s" % (target, self.userid)

        # for index, info in entity list
        for index in self.entities:
            # check if targetname is corresponding
            if es.entitygetvalue(index, "targetname") == targetname:
                entity = index
                # restore last targetname
                es.entitysetvalue(index, "targetname", self.entities[index][1])
                break
            # remove player from dict
        del players[self.userid]

        # if target is "entity"
        if target == "entity":
            self.entity = entity
            # call function with "userid" and "index" parameter
            if callable(self.callback): self.callback(int(self.userid), self.entity)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                es.set("playerview_userid", self.userid)
                es.set("playerview_index", self.entity)
                # execute block
                es.doblock(self.block)

        # if target is "player"
        elif target == "entity":
            # get userid of aimed player
            self.player = es.getuserid(es.gethandlefromindex(self.index))
            self.playerindex = entity if self.player else self.playerindex = -1

            # call function with "userid" and "playerid" parameter
            if callable(self.callback): self.callback(int(self.userid), self.player)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                es.set("playerview_userid", self.userid)
                es.set("playerview_playerid", self.player)
                es.set("playerview_playerindex", self.entity)
                # execute block
                es.doblock(self.block)

    def i_command(self):
        if not es.exists("userid", self.userid): es.dbgmsg(0, "playerview: userid <%s> was not found" % self.userid)
        elif not players.has_key(self.userid):
            # refresh entity list
            self.i_entities()
            # add object in players dict
            players[self.userid] = self

    def entity(self):
        # check user, refresh entity list, add object in players dict
        self.i_command()
        # set entity targetname, exec internal server command
        es.server.insertcmd("es_xentsetname %s _playerview_entity_%s;_playerview_cmd %s entity" % (self.userid, self.userid, self.userid))

    def player(self):
        # check user, refresh entity list, add object in players dict
        self.i_command()
        # set entity targetname, exec internal server command
        es.server.insertcmd("es_xentsetname %s _playerview_player_%s;_playerview_cmd %s player" % (self.userid, self.userid, self.userid))



def entity(userid, callback):
    # playerviewlib.entity(<userid>, <callback>)
    PlayerView(userid, callback).entity()

def player(userid, callback):
    # playerviewlib.player(<userid>, <callback>)
    PlayerView(userid, callback).player()

def es_map_start(event_var):
    # clear players dict
    players.clear()
# registering es_map_start event
es.addons.registerForEvent(__import__(__name__), "es_map_start", es_map_start)




""" fisrt method: call "entity" or "player" function of playerviewlib
def player_say(event_var):
    if event_var["text"] == "!view":
        playerviewlib.entity(event_var["userid"], player_view)

def player_view(userid, index):
    es.tell(userid, "The index of entity that you are looking at is : %s" % index)
"""


""" second method: store PlayerView object for use it anytime
def player_say(event_var):
    if event_var["text"] == "!view":
        # use "entity" or "player" method anytime
        player.player()

def player_connect(event_var):
    global player
    # store PlayerView object once
    player = playerviewlib.PlayerView(event_var["userid"], player_view)

def player_view(userid, playerid):
    es.tell(userid, "The userid of the player that you looking at is : %s" % playerid)
"""
