# -*- coding:Utf-8 -*-
from simplemenu_line import SimpleLine
from simplemenu_base import SimpleBase
import psyco, es, sys, popuplib2, simplemenulib2, simplemenu_tool
psyco.full()

class SimpleMenu(SimpleBase, SimpleLine):
    def __init__(self, name, defaultlang, separators, reversenav):
        SimpleBase.__init__(self, reversenav, defaultlang, separators)
        self.name = name
        self.type = "menu"

        self.p_pages = []
        self.p_lines = [""]
        self.p_totalpage = 0
        self.p_multilang = False
        self.p_constructed = False
        self.p_menulang = {
            "_smlib_page" : {"en" : "      ($page/$total)", "fr" : "      ($page/$total)"},
            "_smlib_next" : {"en" : "Next page", "fr" : "Page suivante"},
            "_smlib_back" : {"en" : "Previous page", "fr" : "Page précédente"},
            "_smlib_exit" : {"en" : "Cancel", "fr" : "Fermer le menu"}}

    def __del__(self):
        for popup in self.p_pages: del popup

    def i_select(self, args):
        userid, index, page = args["userid"], args["choice"], args["page"]
        values, navfunc = args["values"], args["navfunc"]

        callfunc, submenu = False, False
        if values.has_key(index):
            line = values[index]
            userargs = { "userid" : userid, "choice" : index, "item" : line.l_item, "value" : line.l_value, "popup" : self.name,
                         "popupid" : self.name, "page" : page, "modifieditem" : line["modifieditem"]}
            if self.b_select is None: es.dbgmsg(1, 'simplemenulib: select function "%s" is not defined' % value)
            else: callfunc = True

        elif navfunc.has_key(index):
            userargs = {"userid" : userid, "choice" : index, "popup" : self.name, "popupid" : self.name, "page" : page}
            if self.b_selectnc is None: es.dbgmsg(1, 'simplemenulib: selectnc function "%s" is not defined' % value)
            else: callfunc = True
            newpage = navfunc[index]
            if newpage:
                try: submenu = self.p_pages[page + newpage - 1]
                except IndexError: pass

        else:
            userargs = {"userid" : userid, "choice" : index, "popup" : self.name, "popupid" : self.name, "page" : page}
            if self.b_selectic is None: es.dbgmsg(1, 'simplemenulib: selectic function "%s" is not defined' % value)
            else: callfunc = True

        if callfunc:
            try: self.b_select(userargs)
            except Exception:
                sys.excepthook(*sys.exc_info())
                sys.exc_clear()
        if submenu: submenu.send(userid)

    def construct(self):
        indexlist = 1
        totalitem = len(self.p_lines) - 1
        totalpage = simplemenu_tool.getTotalPage(totalitem)
        if len(self.p_pages) > totalpage:
            for popup in self.p_pages[totalpage:]: del popup

        for currentpage in xrange(1, totalpage + 1):
            popup = popuplib2.Popup()
            self.p_pages.append(popup)
            value = popup.menuselect_args["value"] = {}
            navfunc = popup.menuselect_args["navfunc"] = {}
            for sep in self.b_separators[0]: popup.append(sep)
            popup.append(" ")
            for sep in self.b_separators[1]: popup.append(sep)

            for indexitem in xrange(1, simplemenu_tool.getItemsPage(currentpage, totalpage, totalitem) + 1):
                line = self.p_lines[indexlist]
                line.l_page = currentpage
                line.l_index = indexitem
                line.l_listid = indexlist
                line.l_popup = popup
                line.l_modifieditem = None
                popup.append(line["formatline"])
                value[indexitem] = line
                indexlist += 1

            for sep in self.b_separators[2]: popup.append(sep)
            cmppage = cmp(self.p_totalpage, currentpage)
            if self.b_reversenav:
                if cmppage:
                    navfunc[8] = 1
                    popup.append("8. " + self.p_menulang["_smlib_next"][self.b_defaultlang])
                if currentpage == 1:
                    if self.b_returnkey:
                        navfunc[9] = 0
                        popup.append("9. " + self.p_menulang["_smlib_back"][self.b_defaultlang])
                else:
                    navfunc[9] = -1
                    popup.append("9. " + self.p_menulang["_smlib_back"][self.b_defaultlang])

            else:
                if currentpage == 1:
                    if self.b_returnkey:
                        navfunc[8] = 0
                        popup.append("8. " + self.p_menulang["_smlib_back"][self.b_defaultlang])
                elif cmppage > -1:
                    navfunc[8] = - 1
                    popup.append("8. " + self.p_menulang["_smlib_back"][self.b_defaultlang])
                if cmppage:
                    navfunc[9] = 1
                    popup.append("9. " + self.p_menulang["_smlib_next"][self.b_defaultlang])

            navfunc[10] = 0
            popup.append("0. " + self.p_menulang["_smlib_exit"][self.b_defaultlang])
            popup.menuselect_args = {"page" : currentpage, "values" : value, "navfunc" : navfunc}
            popup.menuselect = self.i_select
            for sep in self.b_separators[3]: popup.append(sep)
        self.i_title()
        self.p_constructed = True

    def title(self, title, options = {}, showpage = "auto"):
        self.b_showpage = bool(showpage) if showpage <> "auto" else showpage
        self.p_lines[0] = SimpleLine(title, options = options, underline = False)
        if self.p_constructed: self.i_title()

    def additem(self, item, value = "", options = {}, prefix = "", underline = True):
        item, underline = str(item), bool(underline)
        if not self.p_constructed: self.p_lines.append(SimpleLine(item, value, options, prefix, underline))
        else:
            lastline = self.p_lines[-1]

            if line.l_index < 8:
                newline = SimpleLine(item, value, options, prefix, underline)
                newline.l_page = lastline.l_page
                newline.l_index = lastline.l_index + 1
                newline.l_listid = lastline.l_listid + 1
                newline.l_popup = lastline.l_popup

                popup = newline.l_popup
                popup.append(line["formatline"])
                popup.menuselect_args["values"][newline.l_index] = newline.l_index

            else:
                popup = lastline.l_popup

                if self.b_reversenav: keynext, index = 8, lastline.l_listid + self.b_startoptions
                else: keynext, index = 9, lastline.l_listid + self.b_startoptions + 1
                popup.menuselect_args["navfunc"][keynext] = 1
                del popup.menuselect_args["navfunc"][lastline.l_index]
                popup[index] = "%s. %s" % (keynext, self.p_menulang["_smlib_next"][self.b_defaultlang])

                newpopup = popuplib2.Popup()
                lastline.l_page = lastline.l_page + 1
                lastline.l_index = 1
                lastline.l_popup = newpopup
                lastline.l_modifieditem = None

                newline = SimpleLine(item, value, options, prefix, underline)
                newline.l_page = lastline.l_page
                newline.l_index = lastline.l_index + 1
                newline.l_listid = lastline.l_listid + 1
                newline.l_popup = lastline.l_popup
                newpopup.menuselect_args["values"] {1 : lastline, 2 : newline}
                newpopup.menuselect_args["navfunc"][keynext] = 1


    def send(self, users, page = 1):
        useridlist = simplemenu_tool.getUsers(users)
        if not useridlist: es.dbgmsb(1, 'simplemenulib: wrong value for parameter "users"')
        else:
            if not self.p_constructed: self.construct()
            try: page = int(page)
            except ValueError:
                es.dbgmsb(1, 'simplemenulib: wrong value for parameter "page"')
                page = 1

            try:
                popup = self.p_pages[page - 1]
                for userid in useridlist: popup.send(userid)
            except IndexError:
                es.dbgmsb(1, 'simplemenulib: cannot send page "%s" of menu "%s"' % (page, self.name))
                try: 
                    popup = self.p_pages[0]
                    for userid in useridlist: popup.send(userid)
                except IndexError: es.dbgmsb(0, 'simplemenulib: cannot send menu "%s"' % self.name)
